from argparse import ArgumentParser, RawTextHelpFormatter
from colorama import init as colorama_init
from gettext import translation
from humanize import naturalsize
from multiprocessing import cpu_count
from os import system, makedirs
from pydicts import lod
from importlib.resources import files
from unogenerator.commons import __version__, argparse_epilog, addDebugSystem, get_from_process_info, green, red, magenta
from unogenerator.reusing.casts import list2string
from unogenerator.reusing.percentage import Percentage
from socket import socket, AF_INET, SOCK_STREAM
from subprocess import run
from time import sleep

try:
    t=translation('unogenerator', files("unogenerator") / 'locale')
    _=t.gettext
except:
    _=str

def is_port_opened(host, port):
    sock = socket(AF_INET, SOCK_STREAM)
    result = sock.connect_ex((host,port))
    sock.close()
    if result == 0:
       return False
    else:
       return True


def server_start():#!/bin/bash
    parser=ArgumentParser(
        description=_('Launches LibreOffice server to run unogenerator code'), 
        epilog=argparse_epilog(), 
        formatter_class=RawTextHelpFormatter
    )
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('--instances', help=_("Number of concurrent LibreOffice instances"), action="store", type=int,  default=8)
    parser.add_argument('--first_port', help=_("First port assigned to server"), action="store", type=int, default=2002)
    parser.add_argument('--backtrace', help=_("Backtrace logs"), action="store_true", default=False)
    args=parser.parse_args()

    command_start(args.instances, args.first_port, args.backtrace)
    
def command_start(instances, first_port=2002, backtrace=False):
    if not is_port_opened("127.0.0.1", first_port):
        print(red(_("It seems that server is already launched in port {0}").format(first_port)))
        return 

    print(_("Preparing {0} LibreOffice server instances from port {1}:").format(instances, first_port))
    if backtrace is True:
        makedirs("/var/log/unogenerator/", exist_ok=True)
        print(_("You can see instances output in /var/log/unogenerator"))
        backtrace="--backtrace"
    else:
        backtrace=""
    
    for i in range(instances):
        port=first_port+i
        if backtrace is True:
            command=f'loffice --accept="socket,host=localhost,port={port};urp;StarOffice.ServiceManager" -env:UserInstallation=file:///tmp/unogenerator{port} --headless {backtrace} > /var/log/unogenerator/unogenerator.{port}.log 2>&1 &'
        else:
            command=f'loffice --accept="socket,host=localhost,port={port};urp;StarOffice.ServiceManager" -env:UserInstallation=file:///tmp/unogenerator{port} --headless &'
        sleep(0.4)

        print(_("  - Launched LibreOffice headless instance in port {0}").format(port))
        system(command)

def server_stop():
    parser=ArgumentParser(
        description=_('Stops LibreOffice server to finish using unogenerator code'), 
        epilog=argparse_epilog(), 
        formatter_class=RawTextHelpFormatter
    )
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('--debug', help=_("Debug program information"), choices=["DEBUG","INFO","WARNING","ERROR","CRITICAL"], default="ERROR")

    args=parser.parse_args()

    addDebugSystem(args.debug)
    command_stop()
def command_stop():
    command="pkill -c -f ';urp;StarOffice.ServiceManager'"
    s=run(command ,shell=True, capture_output=True)
    print(_("  - Server was stopped killing {0} processes").format(s.stdout.decode('UTF-8').strip()))
    
    
def monitor():    
    colorama_init()
    parser=ArgumentParser(
        description=_('UnoGenerator monitor'), 
        epilog=argparse_epilog(), 
        formatter_class=RawTextHelpFormatter
    )
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('--debug', help=_("Debug program information"), choices=["DEBUG","INFO","WARNING","ERROR","CRITICAL"], default="ERROR")
    parser.add_argument('--restart', help=_("Restart server when idle"), action="store_true", default=False)
    parser.add_argument('--recommended', help=_("Recommended max memory (Mb) to restart server when idle. By default: 600Mb per instance"), action="store", type=int, default=cpu_count()*600)
    args=parser.parse_args()

    addDebugSystem(args.debug)
    command_monitor(args.restart, args.recommended)
    
## @param restart boolean. To restart unogenerator server when idle and used memory above recommended memory
## @param recommended. Integer. Recomended memory in Mb
def command_monitor(restart, recommended_memory):
    ld=get_from_process_info(cpu_percentage=True)
    ld=lod.lod_order_by(ld, "port")
    instances=len(ld)
    max_mem_recommended=recommended_memory*1024*1024
    list_ports=lod.lod2list(ld, 'port', True)
    cpu_nums=lod.lod2list(ld, 'cpu_number', True)
    cpu_percentage=Percentage(lod.lod_average(ld, "cpu_percentage"), 100)
    mem_total=lod.lod_sum(ld, "mem")
    str_mem_total=green(naturalsize(mem_total)) if mem_total<max_mem_recommended else red(naturalsize(mem_total))
    str_cpu_percentage= green(cpu_percentage) if cpu_percentage.value==0 else red(cpu_percentage)

    print(_("Instances: {0}").format(green(instances)))
    print(_("Ports used: {0}").format(green(list2string(list_ports))))
    print(_("CPU used: {0}").format(green(list2string(cpu_nums))))
    print(_("Memory used: {0}").format(str_mem_total))
    print(_("Max memory recommended: {0}".format(green(naturalsize(max_mem_recommended)))))    
    print(_("CPU percentage: {0}".format(str_cpu_percentage)))
    
    total_cons=0
    for d in ld:
        for con in  d["object"].connections():
            if con.status=="ESTABLISHED":
                total_cons=total_cons+1
    
    str_connections= green(total_cons) if total_cons==0 else red(total_cons)
    print(_("Connections: {0}").format(str_connections))
    if restart is True:
        if mem_total>max_mem_recommended:
            if total_cons==0 :
                print(magenta(_("Restarting server")))
                first_port=min(list_ports)
                command_stop()
                sleep(1)
                command_start(instances, first_port, False)
                sleep(10)
                print(magenta(_("Server restarted")))
                command_monitor(False, recommended_memory)
            else:
                print(magenta(_("Server wasn't restarted because it's busy")))
                
        else:
            print(magenta(_("Server doesn't need to be restarted")))
            
## @return Boolean. True if server is working
def is_server_working():
    ld=get_from_process_info(attempts=2)
    if ld==[]:
        return False
    return True



## THIS IS FILE IS FROM https://github.com/turulomio/reusingcode/python/file_functions.py
## IF YOU NEED TO UPDATE IT PLEASE MAKE A PULL REQUEST IN THAT PROJECT AND DOWNLOAD FROM IT
## DO NOT UPDATE IT IN YOUR CODE


## @brief file_functions. Functions used to manipulate text in files

from os import remove


def replace_in_file(filename, s, r):
    data=open(filename,"r").read()
    remove(filename)
    data=data.replace(s,r)
    f=open(filename, "w")
    f.write(data)
    f.close()


## Replace a line in file that contains. Must finish with a \n or change of line
def replace_line_in_file_that_contains(filename, s, r):
    n=""
    with open(filename, "r") as f:
        for line in f.readlines():
            if s in line:
                n=n+r
            else:
                n=n+line
    remove(filename)
    with open(filename, "w") as f:
        f.write(n)



if __name__ == "__main__":
    with open("text.txt", "w") as f:
        f.write("This ia a text\n")
        f.write("This is another text\n")
        f.write("This is poetry\n")
    
    replace_in_file("text.txt", "text", "good text")
    replace_line_in_file_that_contains("text.txt", "another", "This is not another text\n")



from unogenerator.reusing.github import download_from_github
from unogenerator.reusing.file_functions import replace_in_file
from unogenerator import __version__
from os import system
from sys import argv


def pytest():
    system("pytest")
    
def coverage():
    system("coverage run --omit='*/reusing/*,*uno.py' -m pytest && coverage report && coverage html")


def reusing():
    """
        Actualiza directorio reusing
        poe reusing
        poe reusing --local
    """
    local=False
    if len(argv)==2 and argv[1]=="--local":
        local=True
        print("Update code in local without downloading was selected with --local")
    if local==False:
        download_from_github('turulomio','reusingcode','python/github.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/casts.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/datetime_functions.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/file_functions.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/decorators.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/libmanagers.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/percentage.py', 'unogenerator/reusing/')
        download_from_github('turulomio','reusingcode','python/currency.py', 'unogenerator/reusing/')
    
    replace_in_file("unogenerator/reusing/casts.py","from currency","from unogenerator.reusing.currency")
    replace_in_file("unogenerator/reusing/casts.py","from percentage","from unogenerator.reusing.percentage")

def translate():
        system("xgettext -L Python --no-wrap --no-location --from-code='UTF-8' -o unogenerator/locale/unogenerator.pot  unogenerator/*.py unogenerator/reusing/*.py ")
        system("msgmerge -N --no-wrap -U unogenerator/locale/es.po unogenerator/locale/unogenerator.pot")
        system("msgfmt -cv -o unogenerator/locale/es/LC_MESSAGES/unogenerator.mo unogenerator/locale/es.po")
        system("msgfmt -cv -o unogenerator/locale/en/LC_MESSAGES/unogenerator.mo unogenerator/locale/en.po")

    
def documentation():
        system("unogenerator_demo --create")
        system("cp -f unogenerator_documentation_en.odt doc/")
        system("cp -f unogenerator_documentation_en.pdf doc/")
        system("cp -f unogenerator_documentation_es.odt doc/")
        system("cp -f unogenerator_documentation_es.pdf doc/")
        system("cp -f unogenerator_example_en.ods doc/")
        system("cp -f unogenerator_example_en.pdf doc/")
        system("cp -f unogenerator_example_es.ods doc/")
        system("cp -f unogenerator_example_es.pdf doc/")
        system("unogenerator_demo --remove")


def release():
    print("""Nueva versión:
  * Cambiar la versión y la fecha en commons.py
  * Cambiar la versión en pyproject.toml
  * Modificar el Changelog en README
  * poe coverage
  * poe translate
  * linguist
  * poe translate
  * poe documentation
  * git commit -a -m 'unogenerator-{0}'
  * git push
  * Hacer un nuevo tag en GitHub
  * poetry build
  * poetry publish --username --password  
  * Crea un nuevo ebuild de UNOGENERATOR Gentoo con la nueva versión
  * Subelo al repositorio del portage

""".format(__version__))

### Class to define doxygen command
#class Doxygen(Command):
#    description = "Create/update doxygen documentation in doc/html"
#
#    user_options = [
#      # The format is (long option, short option, description).
#      ( 'user=', None, 'Remote ssh user'),
#      ( 'directory=', None, 'Remote ssh path'),
#      ( 'port=', None, 'Remote ssh port'),
#      ( 'server=', None, 'Remote ssh server'),
#  ]
#
#    def initialize_options(self):
#        self.user="root"
#        self.directory="/var/www/html/doxygen/unogenerator/"
#        self.port=22
#        self.server="127.0.0.1"
#
#    def finalize_options(self):
#        pass
#
#    def run(self):
#        print("Creating Doxygen Documentation")
#        os.system("""sed -i -e "41d" doc/Doxyfile""")#Delete line 41
#        os.system("""sed -i -e "41iPROJECT_NUMBER         = {}" doc/Doxyfile""".format(__version__))#Insert line 41
#        os.system("rm -Rf build")
#        os.chdir("doc")
#        os.system("doxygen Doxyfile")      
#        command=f"""rsync -avzP -e 'ssh -l {self.user} -p {self.port} ' html/ {self.server}:{self.directory} --delete-after"""
#        print(command)
#        os.system(command)
#        os.chdir("..")
  
### Class to define uninstall command
#class Uninstall(Command):
#    description = "Uninstall installed files with install"
#    user_options = []
#
#    def initialize_options(self):
#        pass
#
#    def finalize_options(self):
#        pass
#
#    def run(self):
#        if platform.system()=="Linux":
#            os.system("rm -Rf {}/unogenerator*".format(site.getsitepackages()[0]))
#            os.system("rm /usr/bin/unogenerator*")
#        else:
#            os.system("pip uninstall unogenerator")



def docker_build():
    system("docker build --tag turulomio/unogenerator:latest .")

def docker():
    system(f"docker  run  -p 127.0.0.1:2002:2002 -it {argv[1]}")
